# A lot of this was taken from the Linux Rules.make: seh

#
# User configuration section
#
#PPC
RTAIDIR=/usr/src/rtai-1.6

CONFIG_DYN_MM = y
#CONFIG_CHECK  = y
#CONFIG_SOFT_FP = y
INSRT = $(RTAIDIR)/utils/insrt
RMRT  = $(RTAIDIR)/utils/rmrt

ifndef LINUXDIR
LINUXDIR:=/usr/src/linux
else
LINUXDIR:=$(shell (cd $(LINUXDIR);pwd))
endif

-include $(LINUXDIR)/.config

ifdef CONFIG_SMP
	CONFIG_APIC = y
endif
# End of user configuration section

VERSION      = 22
PATCHLEVEL   = 2
SUBLEVEL     = 6
EXTRAVERSION = -pre1

RTAIRELEASE  = $(VERSION).$(PATCHLEVEL).$(SUBLEVEL)$(EXTRAVERSION)

ARCH        := $(shell uname -m | sed -e s/i.86/i386/ -e s/sun4u/sparc64/ -e s/arm.*/arm/ -e s/sa110/arm/)

# standard CFLAGS
HPATH           = $(RTAIDIR)/include
HOSTCC          = gcc
HOSTCFLAGS      = -Wall -Wstrict-prototypes -O2 -fomit-frame-pointer
CROSS_COMPILE   =

# Include the make variables (CC, etc...)
AS              = $(CROSS_COMPILE)as
LD              = $(CROSS_COMPILE)ld
CC              = $(CROSS_COMPILE)gcc
CPP             = $(CC) -E
MODFLAGS        = -DMODULE

CPPFLAGS := -I$(HPATH) -I$(LINUXDIR)/include
CFLAGS   := $(CPPFLAGS) -Wall -Wstrict-prototypes -O2 -fomit-frame-pointer -fno-strength-reduce -pipe
AFLAGS   := -D__ASSEMBLY__ $(CPPFLAGS)

K_CFLAGS  = -D__KERNEL__
CFLAGS   += $(K_CFLAGS)

ifdef CONFIG_SMP
	CFLAGS += -D__SMP__
endif

ifdef CONFIG_SOFT_FP
	CFLAGS += -msoft-float
endif

export  VERSION PATCHLEVEL SUBLEVEL EXTRAVERSION RTAIRELEASE ARCH \
        TOPDIR HPATH HOSTCC HOSTCFLAGS CROSS_COMPILE AS LD CC \
        CPP MODFLAGS PERL LINUXDIR CPPFLAGS CFLAGS AFLAGS \
	INSTALL_MOD_PATH

.PHONY: dummy sub_dirs $(SUB_DIRS)

all:: $(TARGS) sub_dirs

clean::
	@rm -f *.o *.a $(TARGS) $(EXTRA_CLEAN) .[^.]*
ifdef SUB_DIRS
	@for i in $(SUB_DIRS); do $(MAKE) -C $$i $@; done
endif

%.s: %.c
	$(CC) $(CFLAGS) $(EXTRA_CFLAGS) $(CFLAGS_$@) -S $< -o $@

%.i: %.c
	$(CPP) $(CFLAGS) $(EXTRA_CFLAGS) $(CFLAGS_$@) $< > $@

%.o: %.c
	$(CC) $(CFLAGS) $(EXTRA_CFLAGS) $(CFLAGS_$@) -c -o $@ $<
	@ ( \
            echo 'ifeq ($(strip $(subst $(comma),:,$(CFLAGS) $(EXTRA_CFLAGS) $(CFLAGS_$@))),$$(strip $$(subst $$(comma),:,$$(CFLAGS) $$(EXTRA_CFLAGS) $$(CFLAGS_$@))))' ; \
            echo 'FILES_FLAGS_UP_TO_DATE += $@' ; \
            echo 'endif' \
        ) > $(dir $@)/.$(notdir $@).flags

%.o: %.s
	$(AS) $(ASFLAGS) $(EXTRA_ASFLAGS) -o $@ $<

# make a userspace executable
%: %.c
	$(CC) $(CPPFLAGS) -O2 -o $@ $<

linux_config: 
	@if [ ! -s $(LINUXDIR)/.config ] ; then \
	echo "You need to configure your Linux kernel before building RTAI." ;\
	exit 1 ; \
	fi

ifndef CONFIG_RTHAL
	@echo;\
	echo "!!! ATTENTION: IMPORTANT KERNEL CONFIG PARAMETER MISSING !!!";\
	echo "To build RTAI, you must first select:";\
	echo "Processor type and features -> Real-Time Hardware Abstraction Layer";\
	echo "when configuring your Linux kernel.";\
	echo
	@sleep 5
endif

$(MAKE_SUB_DIRS):
	mkdir $@

sub_dirs: $(SUB_DIRS)
$(SUB_DIRS):
	$(MAKE) -C $@

# This make dependencies quickly
fastdep: dummy
	$(TOPDIR)/utils/mkdep $(wildcard *.[chS]) > .depend
ifdef SUB_DIRS
	$(MAKE) $(patsubst %,_sfdep_%,$(SUB_DIRS)) _FASTDEP_SUB_DIRS="$(SUB_DIRS)"
endif

ifdef _FASTDEP_SUB_DIRS
$(patsubst %,_sfdep_%,$(_FASTDEP_SUB_DIRS)):
	$(MAKE) -C $(patsubst _sfdep_%,%,$@) fastdep
endif

-include .depend
-include $(RTAIDIR)/.hdepend


