#include <linux/module.h>
#include <asm/io.h>
#include <rtai.h>
#include <rtai_fifos.h>
#include "musical_notes.h"
#include "tune.h"



#define FREQ 153000	/* We emit a signal of frequency 153kHz */
#define TICK 3268	/* That's the number of ns between ticks to get called
			   approx. 306000 times a second, which allows us to
			   toggle the parallel port lines 306000 / 2 = 153000
			   times per second */

#define APIC_TIMER 0
#define TIMER_FREQ FREQ_8254
#define FLOAT float

#define LPT 0x378

#define HUNDREDTH_SEC_CYCLES FREQ/100;


static unsigned char lptoutval=0x00;
static int note_half_period_cycles[100];
static int note_full_period_cycles[100];

int note_index=-1;
int note;
int delay=0;

static int emit_carrier=1;
static int note_divider=0;
static int delay_divider=HUNDREDTH_SEC_CYCLES;


static void rt_timer_tick(void)
{
  /* Get a new note and new delay if the current delay is exhausted. Loop if
     we reached the end of the tune */
  if(!delay)
  {
    note_index++;
    note=tune[note_index][0];
    delay=tune[note_index][1];
    if(note==-1 && delay==-1)
    {
      note_index=0;
      note=tune[note_index][0];
      delay=tune[note_index][1];
    }
  }  

  /* Modulate the carrier with a note */
  emit_carrier=((note_divider++)<note_half_period_cycles[note])?1:0;
  note_divider=(note_divider<note_full_period_cycles[note])?note_divider:0;

  /* Toggle the parallel port or not */
  outb(emit_carrier || note==-1?lptoutval^=0xff:0,LPT);
  
  /* Decrement the hundredth second delay divider */
  if(! (delay_divider--))
  {
    delay--;
    delay_divider=HUNDREDTH_SEC_CYCLES;
  }

  rt_times.tick_time=rt_times.intr_time;
  rt_times.intr_time=rt_times.tick_time+rt_times.periodic_tick;
  if (rt_times.tick_time>=rt_times.linux_time)
  {
    rt_times.linux_time+=rt_times.linux_tick;
    if (!APIC_TIMER)
      rt_pend_linux_irq(0);
  } 
}



int init_module(void)
{
  int i;

  /* First, prepare the number of periods and half periods to generate the
    musical notes */
  for(i=0;i<100;i++)
  {
    note_full_period_cycles[i]=FREQ/note_freq[i];
    note_half_period_cycles[i]=FREQ/(note_freq[i]*2);
  }

  rt_mount_rtai();
  rt_request_timer(rt_timer_tick,imuldiv(TICK, TIMER_FREQ, NANO),APIC_TIMER); 
  return 0;
}



void cleanup_module(void)
{
  rt_free_timer();
  rt_umount_rtai();
  return;
}
